@@ -38,7 +38,17 @@ class overlay_interface(object):
         return (self.node_id, self.interface_id) == (other.node_id, other.interface_id)
 
     def __nonzero__(self):
-        return len(self._interface) > 0  # if interface data set
+        #TODO: work out why description and type being set/copied to each overlay
+        try:
+            interface = self._interface
+        except KeyError:
+            return False
+
+        return len(interface) > 0  # if interface data set
+
+    def __lt__(self, other):
+        #TODO: check how is comparing the nodes
+        return ((self.node, self.interface_id) < (other.node, other.interface_id))
 
     @property
     def is_bound(self):
@@ -55,7 +65,7 @@ class overlay_interface(object):
 
     @property
     def _node(self):
-        """Return graph the node belongs to"""
+        """Return graph data the node belongs to"""
         return self._graph.node[self.node_id]
 
     @property
@@ -173,6 +183,8 @@ class overlay_interface(object):
 
 @functools.total_ordering
 class OverlayNode(object):
+    """OverlayNode"""
+
     def __init__(self, anm, overlay_id, node_id):
 # Set using this method to bypass __setattr__
         object.__setattr__(self, 'anm', anm)
@@ -181,6 +193,7 @@ class OverlayNode(object):
         object.__setattr__(self, 'node_id', node_id)
 
     def __hash__(self):
+        """"""
         return hash(self.node_id)
 
     def __nonzero__(self):
@@ -192,6 +205,7 @@ class OverlayNode(object):
         return self.interfaces(type="physical")
 
     def __getnewargs__(self):
+        """"""
         return ()
 
     def __getstate__(self):
@@ -199,6 +213,7 @@ class OverlayNode(object):
         return (self.anm, self.overlay_id, self.node_id)
 
     def __setstate__(self, state):
+        """"""
         # Make self.history = state and last_change and value undefined
         (anm, overlay_id, node_id) = state
         object.__setattr__(self, 'anm', anm)
@@ -206,16 +221,29 @@ class OverlayNode(object):
         object.__setattr__(self, 'node_id', node_id)
 
     def __eq__(self, other):
+        """"""
         try:
             return self.node_id == other.node_id
         except AttributeError:
-            return self.node_id == other
+            return self.node_id == other # eg compare Node to label
 
     @property
     def loopback_zero(self):
+        """"""
         return (i for i in self.interfaces("is_loopback_zero")).next()
 
+    @property
+    def physical_interfaces(self):
+        """"""
+        return self.interfaces(type = "physical")
+
+    @property
+    def loopback_interfaces(self):
+        """"""
+        return self.interfaces(type = "loopback")
+
     def __lt__(self, other):
+        """"""
 # want [r1, r2, ..., r11, r12, ..., r21, r22] not [r1, r11, r12, r2, r21, r22]
 # so need to look at numeric part
         self_node_id = self.node_id
@@ -240,19 +268,23 @@ class OverlayNode(object):
 
     @property
     def _next_int_id(self):
+        """"""
 # returns next free interface ID
         for int_id in itertools.count(1):  # start at 1 as 0 is loopback
             if int_id not in self._interfaces:
                 return int_id
 
     # TODO: interface function access needs to be cleaned up
-    def _add_interface(self, description=None, type="physical", **kwargs):
+    def _add_interface(self, description=None, type="physical", *args, **kwargs):
+        """"""
         data = dict(kwargs)
 
         if self.overlay_id != 'phy' and self.phy:
             next_id = self.phy._next_int_id
             self.phy._interfaces[next_id] = {'type': type,
                                              'description': description} 
+            #TODO: fix this workaround for not returning description from phy graph
+            data['description'] = description
         else:
             next_id = self._next_int_id
             data['type'] = type  # store type on node
@@ -262,12 +294,12 @@ class OverlayNode(object):
         return next_id
 
     def add_loopback(self, *args, **kwargs):
-        """Public function to add  a loopback interface"""
+        """Public function to add a loopback interface"""
         self._add_interface(type="loopback", *args, **kwargs)
 
-    def add_interface(self, **kwargs):
+    def add_interface(self,*args,  **kwargs):
         """Public function to add interface"""
-        interface_id = self._add_interface(**kwargs)
+        interface_id = self._add_interface(*args, **kwargs)
         return overlay_interface(self.anm, self.overlay_id, 
                 self.node_id, interface_id)
 
@@ -380,6 +412,15 @@ class OverlayNode(object):
         neighs = self._overlay.neighbors(self)
         return self._overlay.filter(neighs, *args, **kwargs)
 
+    def neighbor_interfaces(self, *args, **kwargs):
+        #TODO: implement filtering for args and kwargs
+        if len(args) or len(kwargs):
+            log.warning("Attribute-based filtering not currently supported" 
+                    " for neighbor_interfaces")
+
+        return iter(edge.dst_int for edge in self.edges())
+
+
     @property
     def label(self):
         """Returns node label (mapped from ANM)"""
@@ -451,6 +492,7 @@ class OverlayEdge(object):
         object.__setattr__(self, 'dst_id', dst_id)
 
     def __eq__(self, other):
+        """"""
         try:
             return (self.src_id, self.dst_id) == (other.src_id, other.dst_id)
         except AttributeError:
@@ -461,9 +503,11 @@ class OverlayEdge(object):
         return "%s: (%s, %s)" % (self.overlay_id, self.src, self.dst)
 
     def __getnewargs__(self):
+        """"""
         return ()
 
     def __getitem__(self, key):
+        """"""
         overlay = OverlayGraph(self.anm, key)
         return overlay.edge(self)
 
@@ -472,6 +516,7 @@ class OverlayEdge(object):
         return (self.anm, self.overlay_id, self.src_id, self.dst_id)
 
     def __lt__(self, other):
+        """"""
         return ((self.src.node_id, self.dst.node_id) < (other.src.node_id, 
             other.dst.node_id))
 
@@ -569,9 +614,11 @@ class OverlayGraphData(object):
         object.__setattr__(self, 'overlay_id', overlay_id)
 
     def __repr__(self):
+        """"""
         return "Data for (%s, %s)" % (self.anm, self.overlay_id)
 
     def dump(self):
+        """"""
         print str(self._graph.graph)
 
     @property
@@ -592,12 +639,14 @@ class OverlayBase(object):
     """Base class for overlays - overlay graphs, subgraphs, projections, etc"""
 
     def __init__(self, anm, overlay_id):
+        """"""
         if overlay_id not in anm.overlay_nx_graphs:
             raise OverlayNotFound(overlay_id)
         self._anm = anm
         self._overlay_id = overlay_id
 
     def __repr__(self):
+        """"""
         return self._overlay_id
 
     @property
@@ -606,9 +655,11 @@ class OverlayBase(object):
         return OverlayGraphData(self._anm, self._overlay_id)
 
     def __contains__(self, n):
+        """"""
         return n.node_id in self._graph
 
     def interface(self, interface):
+        """"""
         return overlay_interface(self._anm, self._overlay_id,
                 interface.node_id, interface.interface_id)
 
@@ -641,10 +692,14 @@ class OverlayBase(object):
             try:
                 if self._graph[src][dst]['edge_id'] == search_id:
                     return OverlayEdge(self._anm, self._overlay_id, src, dst)
+                elif (src, dst) == (src_id, search_id): 
+                    # searching by nodes
+                    return OverlayEdge(self._anm, self._overlay_id, src, dst)
             except KeyError:
                 pass  # no edge_id for this edge
 
     def __getitem__(self, key):
+        """"""
         return self.node(key)
 
     def node(self, key):
@@ -663,6 +718,7 @@ class OverlayBase(object):
             return None
 
     def degree(self, node):
+        """"""
         return node.degree()
 
     def neighbors(self, node):
@@ -675,12 +731,15 @@ class OverlayBase(object):
 
     @property
     def name(self):
+        """"""
         return self.__repr__()
 
     def node_label(self, node):
+        """"""
         return repr(OverlayNode(self._anm, self._overlay_id, node))
 
     def dump(self):
+        """"""
         self._anm.dump_graph(self)
 
     def has_edge(self, edge):
@@ -688,13 +747,16 @@ class OverlayBase(object):
         return self._graph.has_edge(edge.src, edge.dst)
 
     def __iter__(self):
+        """"""
         return iter(OverlayNode(self._anm, self._overlay_id, node)
                     for node in self._graph)
 
     def __len__(self):
+        """"""
         return len(self._graph)
 
     def nodes(self, *args, **kwargs):
+        """"""
         result = self.__iter__()
         if len(args) or len(kwargs):
             result = self.filter(result, *args, **kwargs)
@@ -729,6 +791,7 @@ class OverlayBase(object):
         return result
 
     def filter(self, nbunch=None, *args, **kwargs):
+        """"""
         if not nbunch:
             nbunch = self.nodes()
 
@@ -743,6 +806,7 @@ class OverlayBase(object):
         return (n for n in nbunch if filter_func(n))
 
     def edges(self, src_nbunch=None, dst_nbunch=None, *args, **kwargs):
+        """"""
 # nbunch may be single node
         if src_nbunch:
             try:
@@ -786,7 +850,10 @@ class OverlayBase(object):
 
 
 class OverlaySubgraph(OverlayBase):
+    """OverlaySubgraph"""
+
     def __init__(self, anm, overlay_id, graph, name=None):
+        """"""
         super(OverlaySubgraph, self).__init__(anm, overlay_id)
         self._graph = graph
         self._subgraph_name = name
@@ -800,14 +867,16 @@ class OverlayGraph(OverlayBase):
 
     @property
     def anm(self):
+        """Returns anm for this overlay"""
         return self._anm
 
     @property
     def _graph(self):
-        # access underlying graph for this OverlayNode
+        """Access underlying graph for this OverlayNode"""
         return self._anm.overlay_nx_graphs[self._overlay_id]
 
     def _replace_graph(self, graph):
+        """"""
         self._anm.overlay_nx_graphs[self._overlay_id] = graph
 
     # these work similar to their nx counterparts: just need to strip the
@@ -843,6 +912,7 @@ class OverlayGraph(OverlayBase):
         self._init_interfaces(node_ids)
 
     def add_node(self, node, retain=None, **kwargs):
+        """Adds node to overlay"""
         if not retain:
             retain = []
         try:
@@ -864,6 +934,7 @@ class OverlayGraph(OverlayBase):
         self._init_interfaces([node_id])
 
     def _init_interfaces(self, nbunch=None):
+        """Initialises interfaces"""
         if not nbunch:
             nbunch = [n for n in self._graph.nodes()]
 
@@ -899,7 +970,10 @@ class OverlayGraph(OverlayBase):
         self._init_interfaces()
 
         def numeric_id(edge):
-            return int(edge.edge_id.split("_")[0])
+            try:
+                return int(edge.edge_id.split("_")[0])
+            except ValueError:
+                return edge # can't casr to int
 
         ebunch = sorted(self.edges(), key = numeric_id)
 
@@ -913,9 +987,13 @@ class OverlayGraph(OverlayBase):
             edge._interfaces[dst.id] = dst_int_id
 
     def __delitem__(self, key):
+        """Alias for remove_node. Allows
+        >>> del overlay[node]
+        """
         self.remove_node(key)
 
     def remove_node(self, node):
+        """Removes a node from the overlay"""
         try:
             node_id = node.node_id
         except AttributeError:
@@ -923,6 +1001,7 @@ class OverlayGraph(OverlayBase):
         self._graph.remove_node(node_id)
 
     def add_edge(self, src, dst, retain=None, **kwargs):
+        """Adds an edge to the overlay"""
         if not retain:
             retain = []
         try:
@@ -933,6 +1012,7 @@ class OverlayGraph(OverlayBase):
         self.add_edges_from([(src, dst)], retain, **kwargs)
 
     def remove_edges_from(self, ebunch):
+        """Removes set of edges from ebunch"""
         try:
             ebunch = unwrap_edges(ebunch)
         except AttributeError:
@@ -940,6 +1020,7 @@ class OverlayGraph(OverlayBase):
         self._graph.remove_edges_from(ebunch)
 
     def add_edges(self, *args, **kwargs):
+        """Adds a set of edges. Alias for add_edges_from"""
         self.add_edges_from(args, kwargs)
 
     def add_edges_from(self, ebunch, bidirectional=False,
@@ -1001,13 +1082,16 @@ class OverlayGraph(OverlayBase):
                 edge.set(key, value)
 
     def subgraph(self, nbunch, name=None):
+        """"""
         nbunch = (n.node_id for n in nbunch)  # only store the id in overlay
         return OverlaySubgraph(self._anm, self._overlay_id, 
                 self._graph.subgraph(nbunch), name)
 
 
 class AbstractNetworkModel(object):
+    """"""
     def __init__(self):
+        """"""
         self._overlays = {}
         self.add_overlay("phy")
         self.add_overlay("graphics")
@@ -1018,10 +1102,12 @@ class AbstractNetworkModel(object):
         self.timestamp = time.strftime("%Y%m%d_%H%M%S", time.localtime())
 
     def __repr__(self):
+        """"""
         return "ANM %s" % self.timestamp
 
     @staticmethod
     def __getnewargs__():
+        """"""
         return ()
 
     def __getstate__(self):
@@ -1030,9 +1116,11 @@ class AbstractNetworkModel(object):
 
     @property
     def overlay_nx_graphs(self):
+        """"""
         return self._overlays
 
     def has_overlay(self, overlay_id):
+        """"""
         return overlay_id in self._overlays
 
     def __setstate__(self, state):
@@ -1044,6 +1132,7 @@ class AbstractNetworkModel(object):
         self._build_node_label()
 
     def save(self):
+        """"""
         import autonetkit.ank_json as ank_json
         import os
         import gzip
@@ -1077,6 +1166,7 @@ class AbstractNetworkModel(object):
         self.restore(latest_file)
 
     def restore(self, pickle_file):
+        """"""
         import json
         import gzip
         import autonetkit.ank_json as ank_json
@@ -1089,6 +1179,7 @@ class AbstractNetworkModel(object):
 
     @property
     def _phy(self):
+        """"""
         return OverlayGraph(self, "phy")
 
     def initialise_graph(self, graph):
@@ -1131,12 +1222,15 @@ class AbstractNetworkModel(object):
         return overlay
 
     def overlays(self):
+        """"""
         return self._overlays.keys()
 
     def devices(self, *args, **kwargs):
+        """"""
         return self._phy.filter(*args, **kwargs)
 
     def __getitem__(self, key):
+        """"""
         return OverlayGraph(self, key)
 
     def node_label(self, node):
@@ -1144,6 +1238,7 @@ class AbstractNetworkModel(object):
         return self.default_node_label(node)
 
     def _build_node_label(self):
+        """"""
         def custom_label(node):
             return self.label_seperator.join(
                     str(self._overlays['phy'].node[node.node_id].get(val))
@@ -1154,6 +1249,7 @@ class AbstractNetworkModel(object):
         self.node_label = custom_label
 
     def set_node_label(self, seperator, label_attrs):
+        """"""
         try:
             label_attrs.lower()
             label_attrs = [label_attrs]  # was a string, put into list
@@ -1164,6 +1260,7 @@ class AbstractNetworkModel(object):
         self.label_attrs = label_attrs
 
     def dump_graph(self, graph):
+        """"""
         print "----Graph %s----" % graph
         print "Graph"
         print self.dump_graph_data(graph)
@@ -1174,18 +1271,21 @@ class AbstractNetworkModel(object):
 
     @staticmethod
     def dump_graph_data(graph):
+        """"""
         debug_data = dict((key, val)
                           for key, val in sorted(graph._graph.graph.items()))
         return pprint.pformat(debug_data)
 
     @staticmethod
     def dump_nodes(graph):
+        """"""
         debug_data = dict((graph.node_label(node), data)
                           for node, data in (graph._graph.nodes(data=True)))
         return pprint.pformat(debug_data)
 
     @staticmethod
     def dump_edges(graph):
+        """"""
         debug_data = dict(((graph.node_label(src), graph.node_label(dst)),
             data) for src, dst, data in (graph._graph.edges(data=True)))
         return pprint.pformat(debug_data)
