@@ -22,12 +22,67 @@ def dot_to_underscore(instring):
     """Replace dots with underscores"""
     return instring.replace(".", "_")
 
+
 class RouterCompiler(object):
     """Base router compiler"""
     lo_interface = "lo0"
     lo_interface_prefix = "lo"
 # and set per platform
 
+    def ibgp_session_data(self, session, ip_version):
+        # Don't make staticmethod as may want to extend (e.g. in IOS compiler for vpnv4)
+        node = session.src
+        neigh = session.dst
+
+        if ip_version == 4:
+            neigh_ip = neigh['ipv4']
+            use_ipv4 = True
+            use_ipv6 = False
+        elif ip_version == 6:
+            neigh_ip = neigh['ipv6']
+            use_ipv4 = False
+            use_ipv6 = True
+
+        data = {
+            'neighbor': neigh.label,
+            'use_ipv4': use_ipv4,
+            'use_ipv6': use_ipv6,
+            'asn': neigh.asn,
+            'loopback': neigh_ip.loopback,
+            # TODO: this is platform dependent???
+            'update_source': node.loopback_zero.id,
+            }
+        return data
+
+    def ebgp_session_data(self, session, ip_version):
+        node = session.src
+        neigh = session.dst
+        if ip_version == 4:
+            neigh_ip = neigh['ipv4']
+            local_int_ip = session.src_int['ipv4'].ip_address
+            dst_int_ip = session.dst_int['ipv4'].ip_address
+            use_ipv4 = True
+            use_ipv6 = False
+        elif ip_version == 6:
+            neigh_ip = neigh['ipv6']
+            local_int_ip = session.src_int['ipv6'].ip_address
+            dst_int_ip = session.dst_int['ipv6'].ip_address
+            use_ipv4 = False
+            use_ipv6 = True
+
+        data = {
+            'neighbor': neigh.label,
+            'use_ipv4': use_ipv4,
+            'use_ipv6': use_ipv6,
+            'asn': neigh.asn,
+            'loopback': neigh_ip.loopback,
+            'local_int_ip': local_int_ip,
+            'dst_int_ip': dst_int_ip,
+            # TODO: change templates to access from node.bgp.lo_int
+            'update_source': node.loopback_zero.id,
+            }
+        return data
+
     """Base Router compiler"""
     def __init__(self, nidb, anm):
         self.nidb = nidb
@@ -63,14 +118,17 @@ class RouterCompiler(object):
         node.loopback_zero.id = self.lo_interface
         node.loopback_zero.description = "Loopback"
 
+
         for interface in node.physical_interfaces:
             phy_int = self.anm['phy'].interface(interface)
-
-            interface.description = phy_int.description
-
             interface.physical = True
             #TODO: allocate ID in platform compiler
 
+            if not phy_int:
+                # for instance if added as management interface to nidb in compile
+                continue
+                
+            interface.description = phy_int.description
             if node.ip.use_ipv4:
                 ipv4_int = phy_int['ipv4']
                 interface.ipv4_address = ipv4_int.ip_address
@@ -85,33 +143,33 @@ class RouterCompiler(object):
                 interface.ipv6_address = address_prefixlen_to_network(ipv6_int.ip_address,
                         interface.ipv6_subnet.prefixlen)
 
-
         for interface in node.loopback_interfaces:
             #TODO: check if nonzero is different to __eq__
             if interface == node.loopback_zero:
                 continue
             else:
                 #print "here for non zero", interface.id
-                vrf_interface = self.anm['vrf'].interface(interface)
+                phy_int = self.anm['phy'].interface(interface)
+                if node.ip.use_ipv4:
+                    ipv4_int = phy_int['ipv4']
+                    interface.ipv4_address = ipv4_int.loopback
+                    interface.ipv4_subnet = node.loopback_subnet
+                    interface.ipv4_cidr = address_prefixlen_to_network(interface.ipv4_address,
+                            interface.ipv4_subnet.prefixlen)
+
+                if node.ip.use_ipv6:
+                    ipv6_int = phy_int['ipv6']
+#TODO: for consistency, make ipv6_cidr
+                    #interface.ipv6_subnet = ipv6_int.loopback # TODO: do we need for consistency?
+                    interface.ipv6_address = address_prefixlen_to_network(
+                            ipv6_int.loopback, 128)
+
+                # secondary loopbacks
                 #TODO: check why vrf names not showing up for all
                 #print vrf_interface.vrf_name
                 pass
 
             continue
-            #TODO: reinstate this code once return to vrf setup
-            ip_interface = g_ipv4.interface(interface)
-            vrf_interface = self.anm['vrf'].interface(interface)
-            index = index + 1  # loopback0 (ie index 0) is reserved
-            interface_id = "%s%s" % (self.lo_interface_prefix, index)
-            node.interfaces.append(
-                id=interface_id,
-                description=interface.description,
-                ipv4_address=ip_interface.loopback,
-                ipv4_subnet=node.loopback_subnet,
-                vrf_name=vrf_interface.vrf_name,
-            )
-
-            #node.interfaces.sort("id")
 
     def ospf(self, node):
         """Returns OSPF links, also sets process_id
@@ -127,8 +185,21 @@ class RouterCompiler(object):
         node.ospf.ospf_links = []
         added_networks = set()
         for interface in node.physical_interfaces:
+            if interface.exclude_igp:
+                continue # don't configure IGP for this interface
             ipv4_int = g_ipv4.interface(interface)
             ospf_int = g_ospf.interface(interface)
+            if not ospf_int.is_bound:
+                continue # not an OSPF interface
+            try:
+                ospf_cost = int(ospf_int.cost)
+            except TypeError:
+                try:
+                    ospf_cost = netaddr.IPAddress(ospf_int.cost)
+                except TypeError:
+                    log.debug("Using default OSPF cost of 1 for %s on %s" % (ospf_int, node))
+                    ospf_cost = 1 # default
+            interface.ospf_cost = ospf_cost
             network = ipv4_int.subnet
             if (ospf_int and ospf_int.is_bound
                     and network not in added_networks):  # don't add more than once
@@ -138,11 +209,11 @@ class RouterCompiler(object):
                     area=ospf_int.area,
                 )
 
-
     def bgp(self, node):
         phy_node = self.anm['phy'].node(node)
         g_bgp = self.anm['bgp']
         g_ipv4 = self.anm['ipv4']
+        g_ipv6 = self.anm['ipv6']
         asn = phy_node.asn
         node.asn = asn
         node.bgp.ipv4_advertise_subnets = []
@@ -155,68 +226,69 @@ class RouterCompiler(object):
             node.bgp.ipv6_advertise_subnets = g_ipv6.data.infra_blocks.get(
                 asn) or []
 
-        node.bgp.ibgp_neighbors = []
-        node.bgp.ibgp_rr_clients = []
-        node.bgp.ibgp_rr_parents = []
-        node.bgp.ebgp_neighbors = []
+        ibgp_neighbors = []
+        ibgp_rr_clients = []
+        ibgp_rr_parents = []
 
-        def format_session(session, use_ipv4=False, use_ipv6=False):
-            neigh = session.dst
-            if use_ipv4:
-                neigh_ip = g_ipv4.node(neigh)
-            elif use_ipv6:
-                neigh_ip = self.anm['ipv6'].node(neigh)
+
+        g_ibgp_v4 = self.anm['ibgp_v4']
+        for session in g_ibgp_v4.edges(phy_node):
+            if session.exclude:
+                log.debug("Skipping excluded ibgp session %s" % session)
+                continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
+
+            data = self.ibgp_session_data(session, ip_version = 4)
+
+            direction = session.direction
+            if direction == 'down':
+                ibgp_rr_clients.append(data)
+            elif direction == 'up':
+                ibgp_rr_parents.append(data)
             else:
-                log.debug(
-                    "Neither v4 nor v6 selected for BGP session %s, skipping"
-                    % session)
-                return
+                ibgp_neighbors.append(data)
 
-            #TODO: Split out ebgp and ibgp
+        #TODO: check v6 hierarchy only created if node set to being v4 or v6
+        g_ibgp_v6 = self.anm['ibgp_v6']
+        for session in g_ibgp_v6.edges(phy_node):
+            if session.exclude:
+                log.debug("Skipping excluded ibgp session %s" % session)
+                continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
+            data = self.ibgp_session_data(session, ip_version = 6)
 
-            if session.type == "ibgp":
-                data = {
-                    'neighbor': neigh.label,
-                    'use_ipv4': use_ipv4,
-                    'use_ipv6': use_ipv6,
-                    'asn': neigh.asn,
-                    'loopback': neigh_ip.loopback,
-                    # TODO: this is platform dependent???
-                    'update_source': node.loopback_zero.id,
-                }
-                if session.direction == 'down':
-                    # ibgp_rr_clients[key] = data
-                    node.bgp.ibgp_rr_clients.append(data)
-                elif session.direction == 'up':
-                    node.bgp.ibgp_rr_parents.append(data)
-                else:
-                    node.bgp.ibgp_neighbors.append(data)
+            direction = session.direction
+            if direction == 'down':
+                ibgp_rr_clients.append(data)
+            elif direction == 'up':
+                ibgp_rr_parents.append(data)
             else:
-                if use_ipv4:
-                    local_int_ip = session.src_int['ipv4'].ip_address
-                    dst_int_ip = session.dst_int['ipv4'].ip_address
-                elif use_ipv6:
-                    local_int_ip = session.src_int['ipv6'].ip_address
-                    dst_int_ip = session.dst_int['ipv6'].ip_address
+                ibgp_neighbors.append(data)
 
-                # TODO: make this a returned value
-                node.bgp.ebgp_neighbors.append({
-                    'neighbor': neigh.label,
-                    'use_ipv4': use_ipv4,
-                    'use_ipv6': use_ipv6,
-                    'asn': neigh.asn,
-                    'loopback': neigh_ip.loopback,
-                    'local_int_ip': local_int_ip,
-                    'dst_int_ip': dst_int_ip,
-                    # TODO: change templates to access from node.bgp.lo_int
-                    'update_source': node.loopback_zero.id,
-                })
+        #TODO: update this to use ibgp_v4 and ibgp_v6 overlays
 
-        for session in g_bgp.edges(phy_node):
-            if node.ip.use_ipv4:
-                format_session(session, use_ipv4=True)
-            if node.ip.use_ipv6:
-                format_session(session, use_ipv6=True)
+        node.bgp.ibgp_neighbors = ibgp_neighbors
+        node.bgp.ibgp_rr_clients = ibgp_rr_clients 
+        node.bgp.ibgp_rr_parents = ibgp_rr_parents
+
+        #ebgp
+
+        ebgp_neighbors = []
+        g_ebgp_v4 = self.anm['ebgp_v4']
+        for session in g_ebgp_v4.edges(phy_node):
+            if session.exclude:
+                log.debug("Skipping excluded ebgp session %s" % session)
+                continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
+            data = self.ebgp_session_data(session, ip_version = 4)
+            ebgp_neighbors.append(data)
+
+        g_ebgp_v6 = self.anm['ebgp_v6']
+        for session in g_ebgp_v6.edges(phy_node):
+            if session.exclude:
+                log.debug("Skipping excluded ebgp session %s" % session)
+                continue # exclude from regular ibgp config (eg for VRF, VPLS, etc)
+            data = self.ebgp_session_data(session, ip_version = 6)
+            ebgp_neighbors.append(data)
+
+        node.bgp.ebgp_neighbors = ebgp_neighbors
         node.bgp.ebgp_neighbors.sort("asn")
 
         return
@@ -226,6 +298,9 @@ class RouterCompiler(object):
         node.isis.isis_links = []
 
         for interface in node.physical_interfaces:
+            if interface.exclude_igp:
+                continue # don't configure IGP for this interface
+
             phy_int = self.anm['phy'].interface(interface)
 
             isis_int = phy_int['isis']
@@ -257,7 +332,6 @@ class RouterCompiler(object):
                         'use_ipv6': node.ip.use_ipv6,
                         } #TODO: add wrapper for this
 
-
 class QuaggaCompiler(RouterCompiler):
     """Base Quagga compiler"""
     lo_interface = "lo0:1"
@@ -274,10 +348,10 @@ class QuaggaCompiler(RouterCompiler):
         super(QuaggaCompiler, self).interfaces(node)
         # OSPF cost
 
-        if phy_node.is_router:
+        if phy_node.is_l3device:
             node.loopback_zero.id = self.lo_interface
             node.loopback_zero.description = "Loopback"
-            node.loopback_zero.ipv4_address=ipv4_node.loopback,
+            node.loopback_zero.ipv4_address=ipv4_node.loopback
             node.loopback_zero.ipv4_subnet=node.loopback_subnet
 
     def ospf(self, node):
@@ -288,6 +362,9 @@ class QuaggaCompiler(RouterCompiler):
         node.ospf.passive_interfaces = []
 
         for interface in node.physical_interfaces:
+            if interface.exclude_igp:
+                continue # don't configure IGP for this interface
+
             bgp_int = self.anm['bgp'].interface(interface)
             if bgp_int.ebgp: # ebgp interface
                 node.ospf.passive_interfaces.append(
@@ -316,7 +393,18 @@ class IosBaseCompiler(RouterCompiler):
     lo_interface_prefix = "Loopback"
     lo_interface = "%s%s" % (lo_interface_prefix, 0)
 
+    def ibgp_session_data(self, session, ip_version):
+        """Wraps RouterCompiler ibgp_session_data
+        adds vpnv4 = True if ip_version == 4 and session is in g_ibgp_vpn_v4"""
+        data = super(IosBaseCompiler, self).ibgp_session_data(session, ip_version)
+        if ip_version == 4:
+            g_ibgp_vpn_v4 = self.anm['ibgp_vpn_v4']
+            if g_ibgp_vpn_v4.has_edge(session):
+                data['use_vpnv4'] = True
+        return data
+
     def compile(self, node):
+        self.vrf_igp_interfaces(node)
         phy_node = self.anm['phy'].node(node)
 
         if node in self.anm['ospf']:
@@ -332,10 +420,13 @@ class IosBaseCompiler(RouterCompiler):
             self.isis(node)
 
         node.label = self.anm['phy'].node(node).label
+        """Note:
+        VRFs are either: before, and mark IGP interfaces to skip;
+        or after, and remove IGP interfaces
+        """
         self.vrf(node)
 
     def interfaces(self, node):
-
         phy_loopback_zero = self.anm['phy'].interface(node.loopback_zero)
         if node.ip.use_ipv4:
             ipv4_loopback_subnet = netaddr.IPNetwork("0.0.0.0/32")
@@ -363,35 +454,106 @@ class IosBaseCompiler(RouterCompiler):
             node.bgp.ipv6_advertise_subnets = [node.loopback_zero.ipv6_address]
 
         # vrf
+        #TODO: this should be inside vrf section?
         node.bgp.vrfs = []
+
         vrf_node = self.anm['vrf'].node(node)
         if vrf_node.vrf_role is "PE":
+
+            # iBGP sessions for this VRF
+            vrf_ibgp_neighbors = defaultdict(list)
+
+            g_ibgp_v4 = self.anm['ibgp_v4']
+            for session in g_ibgp_v4.edges(vrf_node):
+                if session.exclude and session.vrf:
+                    data = self.ibgp_session_data(session, ip_version = 4)
+                    vrf_ibgp_neighbors[session.vrf].append(data)
+
+            g_ibgp_v6 = self.anm['ibgp_v6']
+            for session in g_ibgp_v6.edges(vrf_node):
+                if session.exclude and session.vrf:
+                    data = self.ibgp_session_data(session, ip_version = 6)
+                    vrf_ibgp_neighbors[session.vrf].append(data)
+
+            # eBGP sessions for this VRF
+            vrf_ebgp_neighbors = defaultdict(list)
+
+            g_ebgp_v4 = self.anm['ebgp_v4']
+            for session in g_ebgp_v4.edges(vrf_node):
+                if session.exclude and session.vrf:
+                    data = self.ebgp_session_data(session, ip_version = 4)
+                    vrf_ebgp_neighbors[session.vrf].append(data)
+
+            g_ebgp_v6 = self.anm['ebgp_v6']
+            for session in g_ebgp_v6.edges(vrf_node):
+                if session.exclude and session.vrf:
+                    data = self.ebgp_session_data(session, ip_version = 6)
+                    vrf_ebgp_neighbors[session.vrf].append(data)
+
             for vrf in vrf_node.node_vrf_names:
                 rd_index = vrf_node.rd_indices[vrf]
-                rd = "%s:%s" % (node.loopback, rd_index)
+                rd = "%s:%s" % (node.asn, rd_index)
                 node.bgp.vrfs.append(
                     vrf=vrf,
                     rd=rd,
                     use_ipv4=node.ip.use_ipv4,
                     use_ipv6=node.ip.use_ipv6,
+                    vrf_ebgp_neighbors = vrf_ebgp_neighbors[vrf],
+                    vrf_ibgp_neighbors = vrf_ibgp_neighbors[vrf],
                 )
 
+        # Retain route_target if in ibgp_vpn_v4 and RR or HRR (set in design)
+        vpnv4_node = self.anm['ibgp_vpn_v4'].node(node)
+        if vpnv4_node:
+            retain = False
+            if vpnv4_node.retain_route_target:
+                retain = True
+            node.bgp.vpnv4 = {'retain_route_target': retain}
+
+    def vrf_igp_interfaces(self, node):
+        # marks physical interfaces to exclude from IGP
+        vrf_node = self.anm['vrf'].node(node)
+        if vrf_node.vrf_role is "PE":
+            for interface in node.physical_interfaces:
+                vrf_int = self.anm['vrf'].interface(interface)
+                if vrf_int.vrf_name:
+                    interface.exclude_igp = True
+
     def vrf(self, node):
         g_vrf = self.anm['vrf']
         vrf_node = self.anm['vrf'].node(node)
         node.vrf.vrfs = []
         if vrf_node.vrf_role is "PE":
+            #TODO: check if mpls ldp already set elsewhere
             for vrf in vrf_node.node_vrf_names:
                 route_target = g_vrf.data.route_targets[node.asn][vrf]
+
                 node.vrf.vrfs.append({
                     'vrf': vrf,
                     'route_target': route_target,
                 })
 
             for interface in node.interfaces:
-                vrf_link = self.anm['vrf'].edge(interface._edge_id)
-                if vrf_link:
-                    interface['vrf'] = vrf_link.vrf # mark interface as being part of vrf
+                vrf_int = self.anm['vrf'].interface(interface)
+                if vrf_int.vrf_name:
+                    interface.vrf = vrf_int.vrf_name # mark interface as being part of vrf
+                    if interface.physical:
+                        interface.description += " vrf %s" % vrf_int.vrf_name
+
+        if vrf_node.vrf_role in ("P", "PE"):
+            # Add PE -> P, PE -> PE interfaces to MPLS LDP
+            node.mpls.ldp_interfaces = []
+            for interface in node.physical_interfaces:
+                mpls_ldp_int = self.anm['mpls_ldp'].interface(interface)
+                if mpls_ldp_int.is_bound:
+                    node.mpls.ldp_interfaces.append(interface.id)
+
+        if vrf_node.vrf_role is "P":
+            node.mpls.ldp_interfaces = []
+            for interface in node.physical_interfaces:
+                node.mpls.ldp_interfaces.append(interface.id)
+
+        vrf_node = self.anm['vrf'].node(node)
 
         node.vrf.use_ipv4 = node.ip.use_ipv4
         node.vrf.use_ipv6 = node.ip.use_ipv6
@@ -404,6 +566,9 @@ class IosBaseCompiler(RouterCompiler):
 
             ospf_int = phy_int['ospf']
             if ospf_int and ospf_int.is_bound:
+                if interface.exclude_igp:
+                    continue # don't configure IGP for this interface
+
                 interface.ospf = {
                         'cost': ospf_int.cost,
                         'area': ospf_int.area,
@@ -428,9 +593,13 @@ class Ios2Compiler(IosBaseCompiler):
         interfaces_by_area = defaultdict(list)
 
         for interface in node.physical_interfaces:
+            if interface.exclude_igp:
+                continue # don't configure IGP for this interface
+
             ospf_int = g_ospf.interface(interface)
             if ospf_int and ospf_int.is_bound:
                 area = ospf_int.area
+                area = str(area) # can't serialize IPAddress object to JSON
                 interfaces_by_area[area].append({
                     'id': interface.id,
                     'cost': int(ospf_int.cost),
@@ -440,6 +609,7 @@ class Ios2Compiler(IosBaseCompiler):
         loopback_zero = node.loopback_zero
         ospf_loopback_zero = g_ospf.interface(loopback_zero)
         router_area = ospf_loopback_zero.area # area assigned to router
+        router_area = str(router_area) # can't serialize IPAddress object to JSON
         interfaces_by_area[router_area].append({
             'id': node.loopback_zero.id,
             'cost': 0,
@@ -491,9 +661,8 @@ class JunosphereCompiler(PlatformCompiler):
         junos_compiler = JunosCompiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='junos'):
             nidb_node = self.nidb.node(phy_node)
-            nidb_node.render.template = "templates/junos.mako"
-            nidb_node.render.dst_folder = "rendered/%s/%s" % (
-                self.host, "junosphere")
+            nidb_node.render.template = os.path.join("templates","junos.mako")
+            nidb_node.render.dst_folder = os.path.join("rendered",self.host,"junosphere")
             nidb_node.render.dst_file = "%s.conf" % ank.name_folder_safe(
                 phy_node.label)
 
@@ -516,23 +685,22 @@ class NetkitCompiler(PlatformCompiler):
         g_phy = self.anm['phy']
         quagga_compiler = QuaggaCompiler(self.nidb, self.anm)
 # TODO: this should be all l3 devices not just routers
-        for phy_node in g_phy.nodes('is_router', host=self.host, syntax='quagga'):
+        for phy_node in g_phy.nodes('is_l3device', host=self.host, syntax='quagga'):
             folder_name = naming.network_hostname(phy_node)
             nidb_node = self.nidb.node(phy_node)
-            nidb_node.render.base = "templates/quagga"
-            nidb_node.render.template = "templates/netkit_startup.mako"
-            nidb_node.render.dst_folder = "rendered/%s/%s" % (
-                self.host, "netkit")
-            nidb_node.render.base_dst_folder = "rendered/%s/%s/%s" % (
-                self.host, "netkit", folder_name)
+            nidb_node.render.base = os.path.join("templates","quagga")
+            nidb_node.render.template = os.path.join("templates","netkit_startup.mako")
+            nidb_node.render.dst_folder = os.path.join("rendered", self.host, "netkit")
+            nidb_node.render.base_dst_folder = os.path.join("rendered", self.host, "netkit", folder_name)
             nidb_node.render.dst_file = "%s.startup" % folder_name
 
 # allocate zebra information
-            nidb_node.zebra.password = "1234"
+            if nidb_node.is_router:
+                nidb_node.zebra.password = "1234"
             hostname = folder_name
             if hostname[0] in string.digits:
                 hostname = "r" + hostname
-            nidb_node.zebra.hostname = hostname  # can't have . in quagga hostnames
+            nidb_node.hostname = hostname  # can't have . in quagga hostnames
             nidb_node.ssh.use_key = True  # TODO: make this set based on presence of key
 
             # Note this could take external data
@@ -548,9 +716,10 @@ class NetkitCompiler(PlatformCompiler):
             quagga_compiler.compile(nidb_node)
 
             # TODO: move these into inherited BGP config
-            nidb_node.bgp.debug = True
-            static_routes = []
-            nidb_node.zebra.static_routes = static_routes
+            if nidb_node.bgp:
+                nidb_node.bgp.debug = True
+                static_routes = []
+                nidb_node.zebra.static_routes = static_routes
 
         # and lab.conf
         self.allocate_tap_ips()
@@ -558,6 +727,7 @@ class NetkitCompiler(PlatformCompiler):
 
     def allocate_tap_ips(self):
         # TODO: take tap subnet parameter
+        #TODO: this should be tuple of self.host and platform
         lab_topology = self.nidb.topology[self.host]
             # TODO: also store platform
         from netaddr import IPNetwork
@@ -573,9 +743,8 @@ class NetkitCompiler(PlatformCompiler):
     def lab_topology(self):
 # TODO: replace name/label and use attribute from subgraph
         lab_topology = self.nidb.topology[self.host]
-        lab_topology.render_template = "templates/netkit_lab_conf.mako"
-        lab_topology.render_dst_folder = "rendered/%s/%s" % (
-            self.host, "netkit")
+        lab_topology.render_template = os.path.join("templates","netkit_lab_conf.mako")
+        lab_topology.render_dst_folder = os.path.join("rendered", self.host, "netkit")
         lab_topology.render_dst_file = "lab.conf"
         lab_topology.description = "AutoNetkit Lab"
         lab_topology.author = "AutoNetkit"
@@ -593,7 +762,7 @@ class NetkitCompiler(PlatformCompiler):
 
 # TODO: sort this numerically, not just by string
         lab_topology.machines = " ".join(alpha_sort(naming.network_hostname(phy_node)
-                                                    for phy_node in subgraph.nodes("is_l3device")))
+            for phy_node in subgraph.nodes("is_l3device")))
 
         lab_topology.config_items = []
         for node in sorted(subgraph.nodes("is_l3device")):
@@ -648,6 +817,12 @@ class CiscoCompiler(PlatformCompiler):
         for x in itertools.count(0):
             yield "GigabitEthernet0/%s" % x
 
+    @staticmethod
+    def interface_ids_ra():
+        #TODO: make this skip if in list of allocated ie [interface.name for interface in node]
+        for x in itertools.count(0):
+            yield "GigabitEthernet%s" % x
+
     @staticmethod
     def interface_ids_nxos():
         for x in itertools.count(0):
@@ -668,8 +843,6 @@ class CiscoCompiler(PlatformCompiler):
             yield "GigabitEthernet0/0/0/%s" % x
 
     def compile(self):
-        G_in = self.anm['input']
-        specified_int_names = G_in.data.specified_int_names
 #TODO: need to copy across the interface name from edge to the interface
         g_phy = self.anm['phy']
 
@@ -678,10 +851,9 @@ class CiscoCompiler(PlatformCompiler):
         now = datetime.now()
         if settings['Compiler']['Cisco']['timestamp']:
             timestamp = now.strftime("%Y%m%d_%H%M%S_%f")
-            dst_folder = "rendered/%s_%s/%s" % (
-                self.host, timestamp, "cisco")  # TODO: use os.path.join
+            dst_folder = os.path.join("rendered", self.host, timestamp, "cisco")
         else:
-            dst_folder = "rendered/%s/%s" % (self.host, "cisco")
+            dst_folder = os.path.join("rendered", self.host, "cisco")
 # TODO: merge common router code, so end up with three loops: routers, ios
 # routers, ios2 routers
 
@@ -695,8 +867,9 @@ class CiscoCompiler(PlatformCompiler):
                     interface.id = loopback_ids.next()
 
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios'):
+            specified_int_names = phy_node.specified_int_names
             nidb_node = self.nidb.node(phy_node)
-            nidb_node.render.template = "templates/ios.mako"
+            nidb_node.render.template = os.path.join("templates","ios.mako")
             if self.to_memory:
                 nidb_node.render.to_memory = True
             else:
@@ -705,8 +878,13 @@ class CiscoCompiler(PlatformCompiler):
                     phy_node)
 
             # Assign interfaces
-            int_ids = self.interface_ids_ios()
-            int_ids.next()  # 0/0 is used for management ethernet
+            if phy_node.device_subtype == "os":
+                int_ids = self.interface_ids_ios()
+            elif phy_node.device_subtype == "ra":
+                int_ids = self.interface_ids_ra()
+                
+            mgmt_int_id = int_ids.next()  # 0/0 is used for management ethernet
+
             for interface in nidb_node.physical_interfaces:
                 if specified_int_names:
                     interface.id = phy_node.interface(interface).name
@@ -715,11 +893,14 @@ class CiscoCompiler(PlatformCompiler):
                     interface.id = int_ids.next()
 
             ios_compiler.compile(nidb_node)
+            mgmt_int = nidb_node.add_interface(management = True)
+            mgmt_int.id = mgmt_int_id
 
         ios2_compiler = Ios2Compiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios2'):
+            specified_int_names = phy_node.specified_int_names
             nidb_node = self.nidb.node(phy_node)
-            nidb_node.render.template = "templates/ios2/router.conf.mako"
+            nidb_node.render.template = os.path.join("templates","ios2","router.conf.mako")
             if self.to_memory:
                 nidb_node.render.to_memory = True
             else:
@@ -738,10 +919,14 @@ class CiscoCompiler(PlatformCompiler):
 
             ios2_compiler.compile(nidb_node)
 
+            mgmt_int_id = "mgmteth 0/0/CPU0/0"
+            mgmt_int = nidb_node.add_interface(management = True)
+            mgmt_int.id = mgmt_int_id
+
         nxos_compiler = NxOsCompiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='nx_os'):
             nidb_node = self.nidb.node(phy_node)
-            nidb_node.render.template = "templates/nx_os.mako"
+            nidb_node.render.template = os.path.join("templates","nx_os.mako")
             if self.to_memory:
                 nidb_node.render.to_memory = True
             else:
@@ -758,8 +943,38 @@ class CiscoCompiler(PlatformCompiler):
                 else:
                     interface.id = int_ids.next()
 
+            mgmt_int_id = "mgmt0"
+            mgmt_int = nidb_node.add_interface(management = True)
+            mgmt_int.id = mgmt_int_id
             nxos_compiler.compile(nidb_node)
 
+        # assign management IPs
+        #TODO: make this a module
+        lab_topology = self.nidb.topology[self.host]
+        oob_management_ips = {}
+        from netaddr import IPNetwork
+        try:
+            management_subnet = address_prefixlen_to_network(g_phy.data.management_subnet,
+                    g_phy.data.management_prefixlen)
+        except ValueError:
+            management_subnet = IPNetwork("172.16.254.0/16")
+            log.info("Unable to create management subnet: %s/%s, using default of %s" % 
+                    (g_phy.data.management_subnet,
+                    g_phy.data.management_prefixlen, management_subnet))
+        management_ips = management_subnet.iter_hosts()
+        oob_host_ip = management_ips.next() # consume first ip for the host
+        oob_management_ips["host"] = oob_host_ip
+        for nidb_node in sorted(self.nidb.nodes('is_router', host=self.host)):
+            for interface in nidb_node.physical_interfaces:
+                if interface.management:
+                    interface.description = "OOB Management"
+                    interface.ipv4_address = management_ips.next()
+                    interface.ipv4_subnet = management_subnet
+                    interface.ipv4_cidr = address_prefixlen_to_network(interface.ipv4_address, management_subnet.prefixlen)
+                    interface.physical = True
+                    oob_management_ips[str(nidb_node)] = interface.ipv4_address
+        lab_topology.oob_management_ips = oob_management_ips
+
         other_nodes = [phy_node for phy_node in g_phy.nodes('is_router', host=self.host)
                        if phy_node.syntax not in ("ios", "ios2")]
         for node in other_nodes:
@@ -797,7 +1012,7 @@ class DynagenCompiler(PlatformCompiler):
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios'):
             nidb_node = self.nidb.node(phy_node)
             graphics_node = G_graphics.node(phy_node)
-            nidb_node.render.template = "templates/ios.mako"
+            nidb_node.render.template = os.path.join("templates", "ios.mako")
             nidb_node.render.dst_folder = os.path.join(
                 "rendered", self.host, "dynagen", self.config_dir)
             nidb_node.render.dst_file = "%s.cfg" % ank.name_folder_safe(
@@ -832,9 +1047,8 @@ class DynagenCompiler(PlatformCompiler):
     def lab_topology(self):
 # TODO: replace name/label and use attribute from subgraph
         lab_topology = self.nidb.topology[self.host]
-        lab_topology.render_template = "templates/dynagen.mako"
-        lab_topology.render_dst_folder = "rendered/%s/%s" % (
-            self.host, "dynagen")
+        lab_topology.render_template = os.path.join("templates","dynagen.mako")
+        lab_topology.render_dst_folder = os.path.join("rendered", self.host, "dynagen")
         lab_topology.render_dst_file = "topology.net"
 
         lab_topology.config_dir = self.config_dir
