@@ -3,6 +3,7 @@
 from autonetkit.nidb import NIDB
 import autonetkit.render as render
 import random
+from autonetkit import update_http
 import traceback
 from datetime import datetime
 import os
@@ -37,7 +38,6 @@ def file_monitor(filename):
             yield True
         yield False
 
-
 def manage_network(input_graph_string, timestamp, build_options, reload_build=False, grid = None):
     """Build, compile, render network as appropriate"""
     # import build_network_simple as build_network
@@ -46,8 +46,6 @@ def manage_network(input_graph_string, timestamp, build_options, reload_build=Fa
 # remap?
         build_network = reload(build_network)
 
-    messaging = ank_messaging.AnkMessaging()
-
     if build_options['build']:
         if input_graph_string:
             graph = build_network.load(input_graph_string)
@@ -56,16 +54,18 @@ def manage_network(input_graph_string, timestamp, build_options, reload_build=Fa
 
         anm = build_network.build(graph)
         if not build_options['compile']:
-            # publish without nidb
-            body = ank_json.dumps(anm)
-            messaging.publish_compressed("www", "client", body)
+            update_http(anm)
+
+        if build_options['validate']:
+            import ank_validate
+            ank_validate.validate(anm)
 
     if build_options['compile']:
         if build_options['archive']:
             anm.save()
         nidb = compile_network(anm)
-        body = ank_json.dumps(anm, nidb)
-        messaging.publish_compressed("www", "client", body)
+
+        update_http(anm, nidb)
         log.debug("Sent ANM to web server")
         if build_options['archive']:
             nidb.save()
@@ -80,8 +80,8 @@ def manage_network(input_graph_string, timestamp, build_options, reload_build=Fa
         anm.restore_latest()
         nidb = NIDB()
         nidb.restore_latest()
-        body = ank_json.dumps(anm, nidb)
-        messaging.publish_compressed("www", "client", body)
+        update_http(anm, nidb)
+        
 
     if build_options['diff']:
         import autonetkit.diff
@@ -101,11 +101,10 @@ def manage_network(input_graph_string, timestamp, build_options, reload_build=Fa
         deploy_network(anm, nidb, input_graph_string)
 
     if build_options['measure']:
-        measure_network(nidb)
+        measure_network(anm, nidb)
 
     log.info("Finished")
 
-
 def parse_options():
     """Parse user-provided options"""
     import argparse
@@ -131,8 +130,8 @@ def parse_options():
                         default=False, help="Compile")
     parser.add_argument(
         '--build', action="store_true", default=False, help="Build")
-    parser.add_argument('--render', action="store_true",
-                        default=False, help="Compile")
+    parser.add_argument('--render', action="store_true", default=False, help="Compile")
+    parser.add_argument('--validate', action="store_true", default=False, help="Validate")
     parser.add_argument('--deploy', action="store_true",
                         default=False, help="Deploy")
     parser.add_argument('--archive', action="store_true", default=False,
@@ -140,7 +139,7 @@ def parse_options():
     parser.add_argument('--measure', action="store_true",
                         default=False, help="Measure")
     parser.add_argument('--webserver', action="store_true", default=False, help="Webserver")
-    parser.add_argument('--grid', type=int, help="Webserver")
+    parser.add_argument('--grid', type=int, help="Grid Size (n * n)")
     arguments = parser.parse_args()
     return arguments
 
@@ -162,6 +161,7 @@ def main():
     build_options = {
         'compile': options.compile or settings['General']['compile'],
         'render': options.render or settings['General']['render'],
+        'validate': options.validate or settings['General']['validate'],
         'build': options.build or settings['General']['build'],
         'deploy': options.deploy or settings['General']['deploy'],
         'measure': options.measure or settings['General']['measure'],
@@ -170,6 +170,7 @@ def main():
         'archive': options.archive or settings['General']['archive'],
     }
 
+
     if options.webserver:
         log.info("Webserver not yet supported, please run as seperate module")
 
@@ -229,8 +230,7 @@ def main():
             # TODO: need to close filehandles for input and output
             log.info("Exiting")
 
-
-def compile_network(anm):
+def create_nidb(anm):
     nidb = NIDB()
     g_phy = anm['phy']
     g_ip = anm['ip']
@@ -243,6 +243,14 @@ def compile_network(anm):
         "collision_domain") if not n.is_switch]  # Only add created cds - otherwise overwrite host of switched
     nidb.add_nodes_from(
         cd_nodes, retain=['label', 'host'], collision_domain=True)
+
+    #TODO: write a helper functions to do this
+    for node in nidb.nodes("collision_domain"):
+        ipv4_node = anm['ipv4'].node(node)
+        node.ipv4_subnet = ipv4_node.subnet
+        node.ipv6_subnet = ipv4_node['ipv6'].subnet
+
+
 # add edges to switches
     edges_to_add = [edge for edge in g_phy.edges()
             if edge.src.is_switch or edge.dst.is_switch]
@@ -252,6 +260,12 @@ def compile_network(anm):
 # TODO: boundaries is still a work in progress...
     nidb.copy_graphics(g_graphics)
 
+    return nidb
+
+def compile_network(anm):
+    nidb = create_nidb(anm)
+    g_phy = anm['phy']
+
     for target, target_data in config.settings['Compile Targets'].items():
         host = target_data['host']
         platform = target_data['platform']
@@ -273,7 +287,7 @@ def compile_network(anm):
     return nidb
 
 
-def deploy_network(anm, nidb, input_graph_string):
+def deploy_network(anm, nidb, input_graph_string = None):
 
     # TODO: make this driven from config file
     log.info("Deploying network")
@@ -317,25 +331,66 @@ def deploy_network(anm, nidb, input_graph_string):
                 netkit_deploy.extract(host, username, tar_file,
                                       config_path, timeout=60, key_filename=key_file)
             if platform == "cisco":
+                #TODO: check why using nklab here
                 cisco_deploy.package(config_path, "nklab")
 
+def collect_sh_ip_route(anm, nidb, start_node = None):
+    if not start_node:
+        start_node = random.choice([n for n in nidb.nodes("is_router")])
 
-def measure_network(nidb):
+    #TODO: move this to another module, eg measure
     import autonetkit.measure as measure
-    log.info("Measuring network")
-    remote_hosts = [node.tap.ip for node in nidb.nodes("is_router")]
-    dest_node = random.choice([n for n in nidb.nodes("is_l3device")])
-    log.info("Tracing to randomly selected node: %s" % dest_node)
-    dest_ip = dest_node.interfaces[0].ipv4_address  # choose random interface on this node
+    import autonetkit.verify as verify
+    import autonetkit
+    command = 'vtysh -c "show ip route"'
+    #TODO: make auto take tap ip if netkit platform node
+    #TODO: auto make put into list if isinstance(remote_hosts, nidb_node)
+    remote_hosts = [start_node.tap.ip]
+    result = measure.send(nidb, command, remote_hosts)
 
-    command = "traceroute -n -a -U -w 0.5 %s" % dest_ip
-    measure.send(nidb, command, remote_hosts, threads = 10)
+    processed = []
+    for line in result:
+        processed.append([anm['ipv4'].node(n) for n in line])
+
+    #TODO: move this into verify module
+    verification_results = verify.igp_routes(anm, processed)
+    processed_with_results = []
+    for line in processed:
+        prefix = str(line[-1].subnet)
+        try:
+            result = verification_results[prefix]
+        except KeyError:
+            result = False # couldn't find prefix
+        processed_with_results.append({
+            'path': line,
+            'verified': result,
+            })
+
+    autonetkit.update_http(anm, nidb)
+    ank_messaging.highlight([], [], processed_with_results)
+
+
+def measure_network(anm, nidb):
+    import autonetkit.measure as measure
+
+    log.info("Measuring network")
+    if 0:
+        remote_hosts = [node.tap.ip for node in nidb.nodes("is_router")]
+        dest_node = random.choice([n for n in nidb.nodes("is_l3device")])
+        log.info("Tracing to randomly selected node: %s" % dest_node)
+        dest_ip = dest_node.interfaces[0].ipv4_address  # choose random interface on this node
+        command = "traceroute -n -a -U -w 0.5 %s" % dest_ip
+        measure.send(nidb, command, remote_hosts, threads = 10)
     # abort after 10 fails, proceed on any success, 0.1 second timeout (quite aggressive)
-    #command = 'vtysh -c "show ip route"'
-    #measure.send(nidb, command, remote_hosts, threads = 5)
-    remote_hosts = [node.tap.ip for node in nidb.nodes(
-        "is_router") if node.bgp.ebgp_neighbors]
-    command = "cat /var/log/zebra/bgpd.log"
+    if 1:
+        collect_sh_ip_route(anm, nidb)
+        
+
+    if 0:
+        #measure.send(nidb, command, remote_hosts, threads = 5)
+        remote_hosts = [node.tap.ip for node in nidb.nodes(
+            "is_router") if node.bgp.ebgp_neighbors]
+        command = "cat /var/log/zebra/bgpd.log"
 
 if __name__ == "__main__":
     try:
